// Holds information about the viewing area

#ifndef GRAPHICS_H
#define GRAPHICS_H

#include "Vector3.h"
#include <allegro.h>

#define MAX_STR_LEN 500

class Graphics  
{
private:

    int colorDepth;
    int gfxMode;

    Vector3 curColor, curColor2, backColor;     // Drawing colors

    BITMAP * drawingSurface;    // Use as a pointer to the current surface
    BITMAP * buffer;            // Create the drawing buffer here
    BITMAP * background;        // Points to a current background image
    bool backgroundLoaded;

public:

    Graphics            (const int width, const int height, const int depth, 
                        const int mode);
    ~Graphics           ();

    void SetColor       (const float x, const float y, const float z);
    void SetColor       (const Vector3 col);
    void SetColor2      (const Vector3 col);
    void SetBackColor   (const Vector3 col);

    void SetColors (const float c1x, const float c1y, const float c1z, 
            const float c2x, const float c2y, const float c2z, const float c3x, 
            const float c3y, const float c3z);

    int GetWidth        (void);
    int GetHeight       (void);

    bool IsUsingBuffer  (void);
    
    void UseBuffer      (const bool yes);
    void Refresh        (void);
    void YieldCPU       (void);

    void ShowMouse      (void);
    void HideMouse      (void);

    void Msg            (const char * msg);
    void ErrMsg         (const char * msg);

    void Clear          (void);
    void ClearArea      (const int x, const int y, const int width,
                        const int height);

    void Rectangle      (const int x, const int y, const int width,
                        const int height);

    void FillRectangle  (const int x, const int y, const int width,
                        const int height);

    void Pixel          (const int x, const int y, const bool useXor);

    void DottedHLine    (const int x, const int y, const int length,
                        const int dotLength, const bool useXor);
    void DottedVLine    (const int x, const int y, const int length,
                        const int dotLength, const bool useXor);

    void DrawImage      (const int x, const int y, const bool useMask,
                        BITMAP * image);
    void DrawImagePart  (const int x, const int y, const int width,
                        const int height, BITMAP * image);

    void Text           (const int x, const int y, const char * text);
    void TransText      (const int x, const int y, const char * text);
    void LayeredTransText   (const int x, const int y, const char * text);
    
    void GetString          (char *returnStr, const int x, const int y,
                            const char *defaultStr,
                            const unsigned int maxLength);

    void GetStringPrompt    (char * returnStr, const int x,
                            const int y, const char * defaultStr,
                            const char * promptStr,
                            const unsigned int maxLength);

    int GetNumberPrompt     (const int x, const int y, const int min,
                            const int max, const int defaultNumber,
                            const char * promptStr,
                            const unsigned int maxLength);

    bool LoadBackground     (const char * filename);

    void DrawBackground     (void);
};

#endif

