#include "GraphicsOpenGL.h"
#include <GL/glut.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glx.h>
#include "colors.h"

//
// Initialize the graphics to a specified size, color depth, and
// windowed/fullscreen. Do not use a drawing buffer by default, but create it
// in memory anyway, allowing its usage later.
//
// Returns true upon success, or false upon error. If an error occurs, an error
// message is inserted into errMsg, which should be at least 500 characters
// long.
//
GraphicsOpenGL::GraphicsOpenGL (const int w, const int h, const char * title)
{
    windowWidth = w;
    windowHeight = h;

    // Create a graphics window capable of double-buffered 3d graphics 
    glutInitDisplayMode             (GLUT_RGBA | GLUT_DOUBLE | GLUT_DEPTH);
    glutInitWindowSize              (windowWidth, windowHeight);
    glutInitWindowPosition          (0, 0);
    windowHandle = glutCreateWindow (title);
    glutSetWindowTitle              (title);
    glutSetWindow                   (windowHandle);

    // Enable depth testing and backface culling
    glEnable    (GL_DEPTH_TEST);
    glCullFace  (GL_BACK);
    glEnable    (GL_CULL_FACE);

    // Set drawing colors
    curColor.SetXYZ     (WHITE);
    curColor2.SetXYZ    (GRAY);
    backColor.SetXYZ    (BLACK);
}



GraphicsOpenGL::~GraphicsOpenGL ()
{
    glFinish ();
    glutDestroyWindow (windowHandle);
}



int GraphicsOpenGL::GetWidth (void)  { return windowWidth; }
int GraphicsOpenGL::GetHeight (void) { return windowHeight; }



void GraphicsOpenGL::SetColor     (const float x, const float y, const float z)
{
    curColor.SetXYZ (x, y, z);
}


void GraphicsOpenGL::SetColor     (const Vector3 col) { curColor = col; }
void GraphicsOpenGL::SetColor2    (const Vector3 col) { curColor2 = col; }
void GraphicsOpenGL::SetBackColor (const Vector3 col) { backColor = col; }



// Sets the current color, the secondary current color, and the background col
void GraphicsOpenGL::SetColors (const float c1x, const float c1y, const float c1z, 
    const float c2x, const float c2y, const float c2z, const float c3x, 
    const float c3y, const float c3z)
{
    curColor.SetXYZ (c1x, c1y, c1z);
    curColor2.SetXYZ (c2x, c2y, c2z);
    backColor.SetXYZ (c3x, c3y, c3z);
}



// void GraphicsOpenGL::ShowMouse (void)     { show_mouse (drawingSurface); }
// void GraphicsOpenGL::HideMouse (void)     { show_mouse (NULL); }



// void GraphicsOpenGL::Refresh (void) { }



// Erase all the pixels on the drawing surface
void GraphicsOpenGL::Clear (void)
{
    glClearColor (backColor.x, backColor.y, backColor.z, 1.0);
}



// Erase all the pixels in a rectangular area
void GraphicsOpenGL::ClearArea (const int x, const int y, const int width,
    const int height)
{
    
}


/*


//
// Draw a horizontal dotted line starting at (x, y) and going to
// (x + length, y). Make the "dots" (i.e. line segments in the dotted line)
// dotLength pixels long. If useXor is enabled, invert the color; if not, draw
// using the current drawing color.
//
void GraphicsOpenGL::DottedHLine (const int startX, const int startY,
    const int length, const int dotLength, const bool useXor)
{
    int x;

    for (x = startX; x < startX + length - 1; x++)
    {
        if (((x - startX) / dotLength) % 3 == 2)
        {
            Pixel (x, startY, useXor);
        }
    }
}



void GraphicsOpenGL::DottedVLine (const int startX, const int startY,
    const int length, const int dotLength, const bool useXor)
{
    int y;

    for (y = startY; y < startY + length - 1; y++)
    {
        if (((y - startY) / dotLength) % 3 == 2)
        {
            Pixel (startX, y, useXor);
        }
    }
}



// Draws a pixel at (x, y), with the optional argument to invert the existing
// color there
void GraphicsOpenGL::Pixel (const int x, const int y, const bool useXor)
{
    int col;

    if (useXor)
    {
        col = getpixel(drawingSurface, x, y) ^ 0x11111111;
        // col = getpixel(drawingSurface, x, y) ^ 0xFFFFFFFF;
    }
    else
    {
        col = makecol ((int)curColor.x, (int)curColor.y, (int)curColor.z);
    }
    
    putpixel (drawingSurface, x, y, col);
}



// Draw a rectangle with given upper left and lower right corners, with the
// current drawing color
void GraphicsOpenGL::Rectangle (const int x, const int y, const int width,
    const int height)
{
    rect (drawingSurface, x, y, x + width, y + height,
        makecol((int)curColor.x, (int)curColor.y, (int)curColor.z));
}



// Draw a filled rectangle with given upper left and lower right corners, with
// the current drawing color
void GraphicsOpenGL::FillRectangle (const int x, const int y, const int width,
    const int height)
{
    rectfill (drawingSurface, x, y, x + width, y + height,
        makecol((int)curColor.x, (int)curColor.y, (int)curColor.z));
}



// Draws the bitmap image onto (x, y) location on the drawing surface. If
// useMask is true, do not draw the transparent color.
void GraphicsOpenGL::DrawImage (const int x, const int y, const bool useMask,
                        BITMAP * image)
{
    if (useMask)
    {
        draw_sprite (drawingSurface, image, x, y);
    }
    else
    {
        blit (image, drawingSurface, 0, 0, x, y, image->w, image->h);
    }
}



// Draws a (width x height) size part of the image onto (x, y). Does NOT ignore
// the transparent color (draws it).
void GraphicsOpenGL::DrawImagePart (const int x, const int y, const int width,
                        const int height, BITMAP * image)
{
    blit (image, drawingSurface, 0, 0, x, y, width, height);
}



// Displays a message on the screen
void GraphicsOpenGL::Text (const int x, const int y, const char * text)
{
    #ifdef ALLEG_420
    textprintf_ex (drawingSurface, font, x, y, makecol ((int) curColor.x,
        (int) curColor.y, (int) curColor.z), BLACK, "%s", text);
    #endif
    #ifndef ALLEG_420
    textprintf (drawingSurface, font, x, y, makecol ((int) curColor.x,
        (int) curColor.y, (int) curColor.z), "%s", text);
    #endif
}



void GraphicsOpenGL::TransText (const int x, const int y, const char * text)
{
    #ifdef ALLEG_420
    textprintf_ex (drawingSurface, font, x, y, makecol ((int) curColor.x,
        (int) curColor.y, (int) curColor.z), -1, "%s", text);
    #endif
    #ifndef ALLEG_420
    textprintf (drawingSurface, font, x, y, makecol ((int) curColor.x,
        (int) curColor.y, (int) curColor.z), "%s", text);
    #endif
}



void GraphicsOpenGL::LayeredTransText (const int x, const int y, const char * text)
{
    #ifdef ALLEG_420
    textprintf_ex (drawingSurface, font, x, y + 1, makecol ((int) curColor2.x,
        (int) curColor2.y, (int) curColor2.z), -1, "%s", text);
    textprintf_ex (drawingSurface, font, x, y, makecol ((int) curColor.x,
        (int) curColor.y, (int) curColor.z), -1, "%s", text);
    #endif
    #ifndef ALLEG_420
    textprintf (drawingSurface, font, x, y + 1, makecol ((int) curColor2.x,
        (int) curColor2.y, (int) curColor2.z), "%s", text);
    textprintf (drawingSurface, font, x, y, makecol ((int) curColor.x,
        (int) curColor.y, (int) curColor.z), "%s", text);
    #endif
}


// Displays a message at the top left of the screen
void GraphicsOpenGL::Msg (const char * msg)
{
    TEXTPRINTF (screen, font, 0, 0, TEXTPRINTF_COL, "%s", msg);
}



// Displays an error message at the top of the screen
void GraphicsOpenGL::ErrMsg (const char * msg)
{
    Msg (msg);
    PAUSE;
    exit (0);
}



// Starting at position (x, y), with string defaultStr, gets a string input
// from the user, inserting it into returnStr
void GraphicsOpenGL::GetString (char * returnStr, const int x, const int y,
    const char * defaultStr, const unsigned int maxLength)
{
    int key;
    char str [MAX_STR_LEN];
    unsigned int curLetter = strlen (defaultStr);
   
    strcpy (str, defaultStr);
    
    clear_keybuf();

    while ((key >> 8) != KEY_ENTER)
    {
        if (keyboard_needs_poll()) poll_keyboard();
       
        // Clear the area where the text goes
        ClearArea (x, y, text_length(font, "_") * maxLength, text_height(font));
        Text (x, y, str);

        if (strlen(str) < maxLength)
        {
            Text (x + text_length (font, str), y, "_");
        }

        Refresh ();

        key = readkey();
        
        if (
            ( ((key >> 8)   >= KEY_A) & ((key >> 8)   <= KEY_Z)) ||
            ( ((key & 0xff) >= '0')   & ((key & 0xff) <= '9'))   || 
            ( ((key & 0xff) == '(' || (key & 0xff) == ')')) ||
            ( ((key & 0xff) == ',' || (key & 0xff) == ',')) ||
            ( ((key & 0xff) == '-' || (key & 0xff) == ' ')) ||
            ( ((key & 0xff) == '!' || (key & 0xff) == '?')) ||
            ( ((key >> 8)   == KEY_STOP)))
        {
            if (curLetter < maxLength)
            {
                str[curLetter++] = (char) key;
                str[curLetter] = '\0';
            }
        }
        else if (((key >> 8) == KEY_BACKSPACE) & (curLetter > 0))
        { 
            curLetter--;
            str[curLetter] = '\0';
        }
    }

    strcpy (returnStr, str);
}



// Get a string from the user, displaying a prompt string first
void GraphicsOpenGL::GetStringPrompt (char * returnStr, const int x,
    const int y, const char * defaultStr, const char * promptStr,
    const unsigned int maxLength)
{
    Text (x, y, promptStr);
    GetString (returnStr, x + text_length (font, promptStr), y, defaultStr,
        maxLength);
}



//
// Get a number from the user, given a prompt string. Enforces boundaries on
// the number and displays a generic error message if the value is out of
// bounds. Returns -1 on error.
//
int GraphicsOpenGL::GetNumberPrompt (const int x, const int y, const int min,
    const int max, const int defaultNumber, const char * promptStr,
    const unsigned int maxLength)
{
    char str    [MAX_STR_LEN];
    char numStr [MAX_STR_LEN];
    int num;

    // Convert the default number to a string and use GetStringPrompt to get
    // a response back
    sprintf (numStr, "%d", defaultNumber);
    GetStringPrompt (str, x, y, numStr, promptStr, maxLength);

    num = atoi (str);

    if (num < min || num > max)
    {
        allegro_message (
            "Error: this number must lie within the range (%d, %d)!",
            min, max);
        return -1;
    }
    else
    {
        return num;
    }
}



bool GraphicsOpenGL::LoadBackground (const char * filename)
{
    // Deallocate the memory taken up by the existing background
    if (backgroundLoaded) destroy_bitmap (background);

    // Load the bitmap file into our object
    background = load_bitmap (filename, NULL);
    backgroundLoaded = true;                    // Where's my GC??
    return (background != NULL);
}



void GraphicsOpenGL::DrawBackground (void)
{
    stretch_blit (background, drawingSurface, 0, 0, background->w,
        background->h, 0, 0, SCREEN_W, SCREEN_H);
}


*/
