#include "Light.h"
#include <GL/glut.h>
#include "math.h"


// Default constructor - assign the values of the origin and a unit vector
Light::Light ()
{
    zeroPos[0] = 0.0;
    zeroPos[1] = 0.0;
    zeroPos[2] = 0.0;
    zeroPos[3] = 1.0;

    dirI[0] = 1.0;
    dirI[1] = 0.0;
    dirI[2] = 0.0;
    dirI[3] = 0.0;
}


// Declares lighting to GL
void Light::Draw ()
{
     // Define some values for light types here
    float ambientLight []       = {0.2, 0.2, 0.2, 0.2};
    float lightAmbient0 []      = {0.3, 0.3, 0.3, 0.0};     // Soft light
    float lightDiffuse0 []      = {1.0, 1.0, 1.0, 0.0};
    float lightSpecular0 []     = {0.8, 0.8, 0.8, 0.8};
    // float lightSpecular0 []     = {1.0, 1.0, 1.0, 0.0};
    float lightShininess0 []    = {70};
    float origin []             = {0, 0, 0, 1};             // Define (0, 0, 0)
    float lightPos []           = {10.0, 5.0, 10.0   };     // Far!
    glShadeModel    (GL_SMOOTH);

    //  Set material lighting values
    glMaterialfv    (GL_FRONT, GL_SPECULAR, lightSpecular0);
    glMaterialfv    (GL_FRONT, GL_SHININESS, lightShininess0);
    glMaterialfv    (GL_FRONT, GL_DIFFUSE,      lightDiffuse0);

    // Enable lighting effects in GL
    glEnable        (GL_LIGHTING);
    glEnable        (GL_NORMALIZE);
    glEnable        (GL_LIGHT0);

    // Set global ambient light
    glLightModelfv  (GL_LIGHT_MODEL_AMBIENT, ambientLight);

    // Set values for light 0
    glLightfv       (GL_LIGHT0, GL_AMBIENT,     lightAmbient0);
    glLightfv       (GL_LIGHT0, GL_POSITION,    lightPos);
    glLightfv       (GL_LIGHT0, GL_DIFFUSE,     lightDiffuse0);
    glLightfv       (GL_LIGHT0, GL_SPECULAR,    lightSpecular0);
}



// Set custom lighting values
void Light::SetLighting (float r, float g, float b)
{
    float ambientLight [4];

    ambientLight [0] = r;
    ambientLight [1] = g;
    ambientLight [2] = b;
    ambientLight [3] = 0.0;

    glLightfv (GL_LIGHT0, GL_AMBIENT, ambientLight);
}



// Adjust the lighting in real time
void Light::AdjustLighting (void)
{
    static float r, g, b;
    static float curAngle = 0.0, spinSpeed = 0.5;
    static const float Lt0spec[4] = {1.0, 1.0, 1.0, 1.0};
    static const float Noemit[4] = {0.0, 0.0, 0.0, 1.0};

    // Adjust the lighting color
    // r += 0.001;
    // g += 0.0025;
    // b += 0.005;
    r = 2.0;            // Fade red-yellow
    g = 1.0;
    b += 0.0025;

    if (r > 2.0) r = 0;
    if (g > 2.0) g = 0;
    if (b > 2.0) b = 0;

    SetLighting (fabs(1.0 - r), fabs(1.0 - g), fabs(1.0 - b));

    // Adjust the lighting position
    curAngle += spinSpeed;
    while (curAngle > 360.0)
    {
        curAngle -= 360.0;
    }
    
    glPushMatrix ();
    glRotatef (curAngle, 0.0, 0.0, 1.0);            // Rotate around z-axis
    glTranslatef (3.0, 0.0, 0.0);                   // Translate outwards

    glMaterialfv(GL_FRONT_AND_BACK, GL_EMISSION, Lt0spec);  // glow (emissive)
    // glutSolidSphere(0.3, 5, 5);
    glMaterialfv(GL_FRONT_AND_BACK, GL_EMISSION, Noemit);   // Turn off emission
    
    glLightfv (GL_LIGHT0, GL_POSITION, zeroPos);    // Position is transformed 
    //glLightfv (GL_LIGHT0, GL_POSITION, dirI);    // Position is transformed 
    glPopMatrix ();
}


