#include "Piece.h"
#include <iostream>



Piece::Piece ()
{
    cells.clear ();
}



// Construct a piece given a string representation of it, where a space is
// an empty cell and any other character is a filled in cell. The string
// should be 64 characters long (4 maps of size 4 wide and 4 tall).
void Piece::Initialize (const int w, const int h, const int r, const char * str)
{
    width = w;
    height = h;
    numRotations = r;

    // Initialize the cells in the piece using the string
    cells.clear ();
    for (int i = 0; i < width * height * numRotations; i++)
    {
        cells.push_back (str[i] != ' ');
    }
}



void Piece::SetLocation (const int x, const int y)  { xCell = x; yCell = y; }
void Piece::IncLocation (const int x, const int y)  { xCell += x; yCell += y; }
void Piece::SetNumRotations (const int n)           { numRotations = n; }
void Piece::SetCurRotation (const int n)            { curRotation = n; }



void Piece::Reset (void)
{
    xCell = 0;
    yCell = -height;
    curRotation = 0;
}



void Piece::Rotate (const int howMuch)
{
    curRotation += howMuch;
    while (curRotation >= 4) curRotation -= 4;
    while (curRotation < 0) curRotation += 4;
}



// Return the cell at (x, y), which should range from (0, 0) to (3, 3) for a
// 4x4 size piece.
bool Piece::GetCell (const int x, const int y)
{
    return cells [x + (y * width) + (curRotation * 16)];
}

bool Piece::GetNonRotatedCell (const int x, const int y)
{
    return cells [x + (y * width)];
}



int Piece::GetXCell (void)                          { return xCell; }
int Piece::GetYCell (void)                          { return yCell; }
int Piece::GetWidth (void)                          { return width; }
int Piece::GetHeight (void)                         { return height; }
int Piece::GetNumRotations (void)                   { return numRotations; }
int Piece::GetCurRotation (void)                    { return curRotation; }



void Piece::Print (void)
{
    int x, y;
    for (y = 0; y < height; y++)
    {
        for (x = 0; x < width; x++)
        {
            if (cells[x + y * width])
            {
                cout << "0";
            }
            else
            {
                cout << "X";
            }
        }
        cout << endl;
    }
}



