// One tetris piece

#ifndef PIECE_H
#define PIECE_H

#include <vector>
using namespace std;


const char TETRIS_PIECES [7][65] =
{
    "    "
    "0000"
    "    "
    "    "
    "  0 " 
    "  0 "
    "  0 "
    "  0 "
    "    "
    "0000"
    "    "
    "    "
    "  0 " 
    "  0 "
    "  0 "
    "  0 ",

    "    "
    " 00 "
    " 00 "
    "    "
    "    "
    " 00 "
    " 00 "
    "    "
    "    "
    " 00 "
    " 00 "
    "    "
    "    "
    " 00 "
    " 00 "
    "    ",

    "    "
    " 000"
    " 0  "
    "    "
    "    " 
    " 00 "
    "  0 "
    "  0 "
    "    " 
    "  0 "
    "000 "
    "    "
    " 0  "
    " 0  "
    " 00 "
    "    ",

    "    "
    "000 "
    "  0 "
    "    "
    "    " 
    " 00 "
    " 0  "
    " 0  "
    "    " 
    " 0  "
    " 000"
    "    "
    "  0 "
    "  0 "
    " 00 "
    "    ",

    "    "
    "  00"
    " 00 "
    "    "
    " 0  " 
    " 00 "
    "  0 "
    "    "
    "    "
    " 00 "
    "00  "
    "    "
    "    "
    " 0  " 
    " 00 "
    "  0 ",
    
    "    "
    "00  "
    " 00 "
    "    "
    "  0 " 
    " 00 "
    " 0  "
    "    "
    "    "
    " 00 "
    "  00"
    "    "
    "    "
    "  0 " 
    " 00 "
    " 0  ",
    
    "    "
    "  0 "
    " 000"
    "    "
    "  0 " 
    " 00 "
    "  0 "
    "    "
    "    "
    " 000"
    "  0 "
    "    "
    "  0 "
    "  00" 
    "  0 "
    "    " 
}; 
     
       
class Piece
{
private:

    int width, height;      // Size of the piece in cells 
    int xCell, yCell;       // Location of the piece in some map
    int numRotations;       // Max times the piece can be rotated
    int curRotation;        // How many times the piece has been rotated
  
    // Four 4x4 arrays to hold the cells 
    vector<bool> cells;

public:

    Piece               (void);
    Piece               (const int w, const int h, const char * str);
 
    // Construct a piece given a string representation of it, where a space is
    // an empty cell and any other character is a filled in cell. The string
    // should be 64 characters long (4 maps of size 4 wide and 4 tall).
    void Initialize     (const int w, const int h, const int r,
                        const char * str);
    
    void SetLocation    (const int x, const int y);
    void IncLocation    (const int x, const int y);
    void SetNumRotations (const int n);
    void SetCurRotation (const int n);
    void Reset          (void);
    void Rotate         (const int howMuch);

    void Print          (void);

    // Whether the cell at zero-based location (x, y), rotated numRotations
    // times 90 degrees clockwise, is filled in
    bool GetCell        (const int x, const int y);
    bool GetNonRotatedCell (const int x, const int y);
    
    int GetXCell        (void);
    int GetYCell        (void);
    int GetWidth        (void);
    int GetHeight       (void);
    int GetNumRotations (void); 
    int GetCurRotation  (void); 
};

#endif

