#include "Timer.h"
#include <time.h>
#include <iostream>
using namespace std;


// Marks the timer as non-initialized by default
Timer::Timer ()
{
    initialized = false;    
}



// Set the timer length on init
Timer::Timer (const double numSeconds)
{
    SetLength (numSeconds);
}



// Starts a timer that triggers every numSeconds
void Timer::SetLength (const double numSeconds)
{
    triggerSeconds = numSeconds;
    initialized = true;
    Reset ();
}



// Delay for numSeconds seconds
void Timer::Delay (const double numSeconds)
{
    SetLength (numSeconds);
    trigger = false;
    while (!HasTriggered());
}



// Start this timer by saving the current number of clock ticks that have
// elapsed since the program start.
void Timer::Reset (void)
{
    gettimeofday (&startTime, NULL);
    trigger = false;
}



void Timer::Trigger (void)
{
    trigger = true;
}



// Returns whether or not triggerSeconds seconds have elapsed since the last
// reset of the timer.
bool Timer::HasTriggered (void)
{
    return (trigger || (GetSecondsElapsed () > triggerSeconds) && initialized);
}



// Returns the number of seconds elapsed since the last reset.
double Timer::GetSecondsElapsed (void)
{
    static struct timeval curTime;      // Temp timeval object to store time in
    static struct timeval diffTime;     // Temp timeval to store time difference

    gettimeofday (&curTime, NULL);
    TimevalSubtract (&diffTime, &curTime, &startTime);
    return (diffTime.tv_sec + (float) diffTime.tv_usec / 1000000);
}



// Code from www.gnu.org.
// Calculate result = x - y for struct timeval values.
// Return 1 if the difference is negative, otherwise 0.
int Timer::TimevalSubtract (struct timeval *result, struct timeval *x,
    struct timeval *y)
{
    // Perform the carry for the later subtraction by updating y.
    if (x->tv_usec < y->tv_usec)
    {
        int nsec = (y->tv_usec - x->tv_usec) / 1000000 + 1;
        y->tv_usec -= 1000000 * nsec;
        y->tv_sec += nsec;
    }
    if (x->tv_usec - y->tv_usec > 1000000)
    {
        int nsec = (x->tv_usec - y->tv_usec) / 1000000;
        y->tv_usec += 1000000 * nsec;
        y->tv_sec -= nsec;
    }
 
    // Compute the time remaining to wait. tv_usec is certainly positive.
    result->tv_sec = x->tv_sec - y->tv_sec;
    result->tv_usec = x->tv_usec - y->tv_usec;
 
    // Return 1 if result is negative.
    return x->tv_sec < y->tv_sec;
}


