// Gets input from the user and responds to it

#ifndef USERINTERFACE_H
#define USERINTERFACE_H

#include "Graphics.h"
#include "Keyboard.h"
#include "Board.h"
#include "Window.h"
#include "Mouse.h"
#include "Piece.h"
#include "Timer.h"

#define MAX_STR_LEN 500
const double MOVE_DELAY = 0.075;    // Move horizontal delay, in seconds
const double DROP_DELAY = 0.035;    // In seconds
const double ROTATE_DELAY = 0.05;   // In seconds
const double BLINK_DELAY = 0.05;    // In seconds
const double LEVEL_FALL_BASE = 0.5; // How fast pieces fall in level 1 (seconds)
const double LEVEL_FALL_INC = 0.1;  // How much faster pieces fall each new lev
const int NUM_BLINKS = 4;
const int NUM_PIECES = 7;
const long DROP_PIECE_SCORE = 10;
const long ERASE_LINE_SCORE = 200;
const int LEVEL_ADVANCE_LINES = 10;
const int LEVEL_X = 75, LEVEL_Y = 40;   // Location of drawing windows
const int LEVEL_WIDTH = 102;
const int NEXT_X = 75, NEXT_Y = 150;
const int NEXT_WIDTH = 102;

#define BACKGROUND_IMAGE_FILENAME_BASE  "background"
#define BACKGROUND_MISSING_ERR          "Could not load background"


class UserInterface  
{
private:

    Graphics * graphics;
    Keyboard * keyboard;
    Board * board;
    Mouse * mouse;
    Window * window;
    Timer * timer;

    // char * baseDirectory;

    int pieceX, pieceY,         // Cell location of the current piece 
        pieceRotation,          // How many times (0-3) the piece is rotated CW
        tileWidth, tileHeight;  // The size in pixels of a tetris tile

    Piece pieces[NUM_PIECES];   // The six types of pieces to choose from
    int curPiece;               // Which piece we are currently using
    int nextPiece;              // Which piece we use next

    int level;
    long score;
    int lines;
    bool showNext;

public:

    UserInterface (
        const int screenWidth,
        const int screenHeight,
        const int colorDepth,
        const int gfxMode,
        const int levelWidth,
        const int levelHeight,
        const int tileWidth_,
        const int tileHeight_);

    void Loop               (void);
    void RespondToKeyboard  (void);

    void CreatePiece        (void);
    void RotatePiece        (void);
    bool MovePiece          (const int x, const int y);
    bool CheckCollisions    (const int x, const int y);
    void PieceToBoard       (const bool draw);
    void SetCell            (const int xCell, const int yCell, const int tile);
    void DrawCell           (const int xPixel, const int yPixel,
                            const int whichCell);
    void GameOver           (void);
    void DrawWindow         (void);
    void CheckForLines      (void);
    void DisplayCommands    (void);

    void LoadBackground     (void);
};

#endif

