#include "Vector3.h"
#include <math.h>

Vector3::Vector3 ()                             { x = y = z = 0; }
Vector3::Vector3 (float X, float Y, float Z)    { x = X; y = Y; z = Z; }

float Vector3::GetX ()          { return x; }
float Vector3::GetY ()          { return y; }
float Vector3::GetZ ()          { return z; }

void Vector3::SetX (float X)    { x = X; }
void Vector3::SetY (float Y)    { y = Y; }
void Vector3::SetZ (float Z)    { z = Z; }

void Vector3::SetXYZ (float X, float Y, float Z)
{
    x = X;
    y = Y;
    z = Z;
}

//
// Linear algebra functions - change this vector by performing an operation
// that includes this vector
//
void Vector3::Add       (Vector3 a)     { x += a.x; y += a.y; z += a.z; }
void Vector3::Subtract  (Vector3 a)     { x -= a.x; y -= a.y; z -= a.z; }
void Vector3::Negate    ()              { x *= -1;  y *= -1;  z *= -1;  }
void Vector3::Scale     (float s)       { x *= s;   y *= s;   z *= s;   }

//
// Linear algebra functions - change this vector by making it equal to the
// result of an operation on other vectors and/or scalars
//
void Vector3::Add       (Vector3 a, Vector3 b)
{
    x = a.x + b.x;
    y = a.y + b.y;
    z = a.z + b.z;
}

void Vector3::Subtract	(Vector3 a, Vector3 b)
{
    x = a.x - b.x;
    y = a.y - b.y;
    z = a.z - b.z;
}

void Vector3::Negate    (Vector3 a)
{ 
    x = -a.x;
    y = -a.y;
    z = -a.z;
}

void Vector3::Scale     (float s, Vector3 a)   
{
    x = a.x * s;
    y = a.y * s;
    z = a.z * s;
}

void Vector3::Cross     (Vector3 a, Vector3 b)	    // This vector = a x b
{
    x = (a.y * b.z) - (a.z * b.y);
    y = (a.z * b.x) - (a.x * b.z);
    z = (a.x * b.y) - (a.y * b.x);
}


// Other linear algebra functions
void Vector3::Normalize  ()		// Normalize this vector
{
    Scale (1.0f / Magnitude(*this));
}


float Vector3::Magnitude (Vector3 a)	// Get magnitude of this vector
{
    return sqrtf ((x * x) + (y * y) + (z * z));
}


float Vector3::Dot       (Vector3 a)	// Get the dot product of this vector
					// and vector a
{
    return (x * a.x) + (y * a.y) + (z * a.z);
}

