#include "Window.h"
#include "Graphics.h"
#include "Board.h"
#include "Vector3.h"
#include <stdio.h>
#include <string.h>

#define MAX_STR_LEN 500



Window::Window             (const int width_, const int height_,
                            const int startXPixel_, const int startYPixel_,
                            const int containerWidth_,
                            const int containterHeight_)
{
    width = width_;
    height = height_;
    startXPixel = startXPixel_;
    startYPixel = startYPixel_;
    containerWidth = containerWidth_;
    containerHeight = containterHeight_;
    startXCell = 0;
    startYCell = 0;
    isTileHelper = false;
    isTileSelector = false;
    drawGrid = true;
}



void Window::UseGrid (const bool yes)   { drawGrid = yes; }



bool Window::GridEnabled (void)         { return drawGrid; }



// Set the pixel of the upper left corner of the viewing window
void Window::SetUpperLeftPixel (const int x, const int y)
{
    startXPixel = x;
    startYPixel = y;
}



// Set the cell in the level to begin drawing
void Window::SetStartCell (const int x, const int y)
{
    startXCell = x;
    startYCell = y;
}



// Sets the size of the area containing the window, in cells
void Window::Resize (const int newContainerWidth, const int newContainerHeight)
{
    containerWidth = newContainerWidth;
    containerHeight = newContainerHeight;

    // If the container has shrunk such that the viewing window now covers
    // an invalid part of the container, reset the viewing window
    if (startXCell + width > newContainerWidth ||
        startYCell + height > newContainerHeight)
    {
        SetStartCell (0, 0);
    }
}



void Window::IncrementStartCell (const int x, const int y)
{
    startXCell += x;
    startYCell += y;

    if (startXCell < 0)
    {
        startXCell = 0;
    }

    if (startYCell < 0)
    {
        startYCell = 0;
    }

    if (startXCell > containerWidth - width)
    {
        startXCell = containerWidth - width;
    }

    if (startYCell > containerHeight - height)
    {
        startYCell = containerHeight - height;
    }
}



void Window::SetTileSelector (const bool yes)   { isTileSelector = yes; }
void Window::SetTileHelper   (const bool yes)   { isTileHelper = yes; }



int Window::GetWidth        (void)  { return width; }
int Window::GetHeight       (void)  { return height; }
int Window::GetStartXCell   (void)  { return startXCell; }
int Window::GetStartYCell   (void)  { return startYCell; }
int Window::GetStartXPixel  (void)  { return startXPixel; }
int Window::GetStartYPixel  (void)  { return startYPixel; }
int Window::GetInfoAreaX    (void)  { return infoAreaX; }
int Window::GetInfoAreaY    (void)  { return infoAreaY; }
int Window::GetPromptAreaX  (void)  { return promptAreaX; }
int Window::GetPromptAreaY  (void)  { return promptAreaY; }
bool Window::IsTileSelector (void)  { return isTileSelector; }
bool Window::IsTileHelper   (void)  { return isTileHelper; }



// Clear the tile area of the window
void Window::Clear (Graphics * graphics, const int tileWidth,
    const int tileHeight)
{
    graphics->ClearArea (startXPixel, startYPixel, width * tileWidth,
        height * tileHeight);
}



// Draw one window of board
void Window::Draw (Graphics * graphics, Board * board, const int tileWidth,
    const int tileHeight)
{
}



// Refresh the information at the bottom
void Window::UpdateInfoArea (Graphics * graphics, Board * board, 
    const int curTile, const bool leftSide)
{
    /*
    int beginX = startXPixel,
        beginY = graphics->GetHeight() - tiles->GetHeight(),
        endX = graphics->GetWidth() - 1,
        endY = graphics->GetHeight() - 1,
        x, y, w;
    char str [MAX_STR_LEN];
    */

    /*
    // Update the locations for the info area location and size, and prompt area
    // location here, becuase we have the information we need to do so
    infoAreaX =         beginX;
    infoAreaY =         beginY;
    infoAreaWidth =     endX - beginX;
    infoAreaHeight =    endY - beginY;
    promptAreaX =       beginX + tiles->GetWidth() + 5 +
                            text_length (font, "Sprite: 999         ");
    promptAreaY =       beginY;
    */

    if (leftSide)
    {
        /*
        // Draw the current tile
        graphics->DrawImagePart (beginX, beginY, tiles->GetWidth(),
            tiles->GetHeight(), artwork->GetImage (curTile));

        // Draw text showing the tile and sprite 
        x = beginX + tiles->GetWidth() + 5;
        y = beginY + text_height(font);
        w = text_length(font, "Sprite: 999"),

        graphics->ClearArea (x, y, w, text_height(font) * 2);

        sprintf (str, "Tile:   %3d", tiles->SpriteToTile(curTile));
        graphics->Text (x, y, str);

        y += text_height(font);
        sprintf (str, "Sprite: %3d", curTile);
        graphics->Text (x, y, str);

        // Draw text indicating if the tile helper is on or off
        strcpy (str, "Tile Helper: ");
        isTileHelper ? strcat (str, "ON ") : strcat (str, "OFF");
        graphics->Text (promptAreaX, promptAreaY + text_height(font), str);
        */
    }
    else
    {
        /*
        // Draw text showing the position and size of the window and level
        w = text_length (font, "Window: (999, 999)-(999, 999)");
        x = endX - w; 
        y = beginY + text_height(font);

        graphics->ClearArea (x, y, w, text_height(font) * 2);

        sprintf (str, "Window: (%d, %d)-(%d, %d)", startXCell, startYCell,
            startXCell + width - 1, startYCell + height - 1);
        graphics->Text (x, y, str);

        y += text_height(font);
        sprintf (str, "Level:  (%d x %d)", level->GetWidth(),
            level->GetHeight());
        graphics->Text (x, y, str);
        */
    }
}



// Must call UpdateInfoArea before calling this
void Window::ClearInfoArea (Graphics * graphics)
{
    graphics->ClearArea (infoAreaX, infoAreaY, infoAreaWidth, infoAreaHeight);
}


