// The part of the level that is viewable on the screen

#ifndef WINDOW_H
#define WINDOW_H

#include "Graphics.h"
#include "Board.h"

class Window  
{
private:

    int width, height;              // Size of the window in cells
    int containerWidth,             // Size of the area containing the window
        containerHeight;            //    in cells
    int startXPixel, startYPixel;   // Location of upper left corner in pixels
    int startXCell, startYCell;     // What cell in the level the window begins
    bool isTileSelector;            // Toggles between tile selector and level
                                    //    editor view modes
    bool drawGrid;                  // Toggles between drawing lines to guide
                                    //    level editing
    bool isTileHelper;              // Toggles whether or not to draw groups of
                                    //    tiles at once
    int infoAreaX, infoAreaY;       // Location of the upper left corner of the
                                    //    window information area
    int infoAreaWidth, infoAreaHeight;  // Size of the info area in pixels
    int promptAreaX, promptAreaY;   // Location of the upper left corner of the
                                    //    text prompting area
    
    // Cell location of the highlighted cell; use (-1, -1) to highlight nothing
    int highlightedX, highlightedY;

public:

    Window                  (const int width_, const int height_,
                            const int startXPixel_, const int startYPixel_,
                            const int containerWidth_,
                            const int containerHeight_);
   
    void IncrementStartCell (const int x, const int y);
    void SetUpperLeftPixel  (const int x, const int y);
    void SetStartCell       (const int x, const int y);
    void Resize             (const int newWidth, const int newHeight);
    void SetTileHelper      (const bool yes);
    void SetTileSelector    (const bool yes);
    void UseGrid            (const bool yes);

    int GetWidth            (void);
    int GetHeight           (void);
    int GetStartXCell       (void);
    int GetStartYCell       (void);
    int GetStartXPixel      (void);
    int GetStartYPixel      (void);
    int GetInfoAreaX        (void);
    int GetInfoAreaY        (void);
    int GetInfoAreaWidth    (void);
    int GetInfoAreaHeight   (void);
    int GetPromptAreaX      (void);
    int GetPromptAreaY      (void);
    bool IsTileHelper       (void);
    bool IsTileSelector     (void);
    bool GridEnabled        (void);
   
    void Clear              (Graphics * graphics, const int tileWidth,
                            const int tileHeight);
    void Draw               (Graphics * graphics, Board * board,
                            const int tileWidth, const int tileHeight);
    void UpdateInfoArea     (Graphics * graphics, Board * board,
                            const int curTile, const bool leftSide);
    void ClearInfoArea      (Graphics * graphics);
};

#endif
